// ************************************************************
// headers.h -- RomSort
// all functions return the size in bytes of the file image
// iBufSize pointer is the size of the header to skip. Pass
// zero if header is not necessary. (FIXME)
//
// note: The iRomType() function is written to be efficient
// at comparing a string with many possibilities.
// Also, Be careful about changing what is skipped with the
// ROMCENTER symbol is defined.
// ************************************************************

#ifdef ROMCENTER
	#define NOSKIPGENHDR
#endif

#include <string.h>
#include <ctype.h>
#include "convert.h"		// format conversion routines
#include "headers.h"		// for PFN definition

PFN GetRomSize[MaxHdr] = {	// function array - MaxHdr is enumerated in the header
	iNullRet,
	iRomSize_Null,			// No Header
	iRomSize_A78,			// Atari 7800
	iRomSize_Lynx,			// Atari Lynx
	iRomSize_iNes,			// NES
	iRomSize_Genesis_BIN,	// Sega Genesis
	iRomSize_Genesis_SMD,	// Sega Genesis
	iRomSize_SNES_Hdr		// SNES Pro Fighter, Super WildCard
};

int iRomType (char *ext) {
	// need hack for GenBinHdr - return -1 for .bin

	int n;
	if(ext[0] == '*')		// you never know...
		ext++;
	if(ext[0] == '.')
		ext++;

	for(n = 0; ext[n]; n++)	// does this cause problems since ext is not constant?
		ext[n] = tolower(ext[n]);

	switch(ext[0]) {
#ifndef ROMCENTER
	case 'a':
		if(!strcmp( ext, "a78"))
			return A78Hdr;
		break;
	case 'l':
		if(!strcmp( ext, "lnx"))
			return LynxHdr;
		if(!strcmp( ext, "lyx"))
			return LynxHdr;
		break;
#endif
	case 'n':
		if(!strcmp( ext, "nes"))
			return iNesHdr;
		break;
	case 'f':
		if(!strcmp( ext, "fig"))
			return SNES_Hdr_FIG;
		break;
	case 's':
		if(!strcmp( ext, "swc"))	// snes smc has no header
			return SNES_Hdr_FIG;	// but could be included here
		if(!strcmp( ext, "smd"))
			return GenSMDHdr;
		break;
	}

	return NullHdr;
}

// ************************************************************
// Invalid type selected, return fail (false, or zero size)

UINT32 iNullRet( UINT8 *pImage, UINT32 *iBufSize ) {
	return 0;
}

// ************************************************************
// No Header - included for *.*, Atari *.bin
// SNES formats with no header - .008, .058, .078
// All SMS files have "TMR SEGA" at 0x7FF0 except rygar?

UINT32 iRomSize_Null( UINT8 *pImage, UINT32 *iBufSize) { 
	UINT32 iTemp = *iBufSize;
	*iBufSize = 0;
	return iTemp;
}

// ************************************************************
// A7800 format, own hack

UINT32 iRomSize_A78( UINT8 *pImage, UINT32 *iBufSize ) { 
	UINT32 iRet = *iBufSize;
	*iBufSize = 0x80;		// 128 bytes of crap
	iRet -= *iBufSize;
	return iRet;
}


// ************************************************************
// Lynx format, own hack
// pImage[5] should contain number of 0x10000 size blocks

UINT32 iRomSize_Lynx( UINT8 *pImage, UINT32 *iBufSize ) {
	UINT32 iTemp = *iBufSize;
//	if( strcmp(pImage, FileTypeInfo[LynxHdr].signature) ) {
		// Jimmy Connors Tennis has no header
//		*iBufSize = 0;
//		return iTemp;
//	} else {
		*iBufSize = 0x40;
		return( (UINT32) pImage[5] * 0x10000);
//	}
}

// ************************************************************
// iNES-format header, Marat Fayzulin (sp?)

UINT32 iRomSize_iNes( UINT8 *pImage, UINT32 *iBufSize) {
	UINT32 iRet;

//	best way, use info from headers
	//	iRet = ( ( pImage[4] << 1 ) + pImage[5] ) << 13;
	//	if ((*iBufSize - iRet) >511) iRet += 512;	//PC10 data is 512 bytes
// most reliable, use size - 16 but this includes PC-10 data too
	iRet = *iBufSize - 16;
	if( iBufSize) *iBufSize = 16;
	return iRet;
}

// ************************************************************
// Genesis BIN header

UINT32 iRomSize_Genesis_BIN( UINT8 *pImage, UINT32 *iBufSize ) {
	UINT32 iRet = *iBufSize;
	// verify file format here
#if defined(NOSKIPGENHDR) || defined(ROMCENTER)
	*iBufSize = 0;
#else
	*iBufSize = 0x200;	// skip 512 byte bin header because of checksum "violations" and other hacked crap
	iRet -= *iBufSize;
#endif // ROMCENTER || NOSKIPGENHDR
	return iRet;
}

// ************************************************************
// Genesis SMD header

UINT32 iRomSize_Genesis_SMD( UINT8 *pImage, UINT32 *iBufSize ) {
	UINT32 iRet = *iBufSize;
	if( SMD2BIN( (unsigned char *) pImage, *iBufSize )){	// format verification done by SMD2BIN
#if defined(NOSKIPGENHDR) || defined(ROMCENTER)
		*iBufSize = 0x200;			// skip SMD header only
#else
		*iBufSize = 0x400;			// skip SMD header and Bin header
#endif // ROMCENTER || NOSKIPGENHDR
		iRet -= *iBufSize;
		return iRet;
	} else
	return 0;
}

// ************************************************************
// SNES formats with 512 byte header - SMC, SWC, SFC, FIG

UINT32 iRomSize_SNES_Hdr( UINT8 *pImage, UINT32 *iBufSize ) {// S?C formats have 512 bytes

	UINT32 iRet;

	UINT32 iMod = *iBufSize % 0x2000;		// iMod should be 512 or 0

	if(iMod == 0x200) {						// 512 byte header
		iRet = *iBufSize - iMod;
	} else if(iMod) {						// error in image or some other format
		iRet = 0;
	} else {								// no header
		iRet = *iBufSize;
	}

	*iBufSize = iMod;
	return iRet;
}
